﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml.Linq;
using System.IO;
using System.ServiceModel;
using System.Data.Services.Client;
using System.Collections;
using System.Windows.Browser;

namespace SilverlightDemo
{
    public partial class Page : UserControl
    {
        #region " Application setup "

        public Page()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Fires when users should be searched.
        /// </summary>
        private void btnSearch_Click(object sender, RoutedEventArgs e)
        {
            // Get search expression
            string expression = txtUserName.Text.Trim();

            // Search by specified method
            if (radASMX.IsChecked.Value)
            {
                SearchByASMX(expression);
            }
            else if (radWCF.IsChecked.Value)
            {
                SearchByWCF(expression);
            }
            else if (radADONet.IsChecked.Value)
            {
                SearchByADONet(expression);
            }
            else if (radWebClient.IsChecked.Value)
            {
                SearchByWebClient(expression);
            }
            else if (radWebRequest.IsChecked.Value)
            {
                SearchByWebRequest(expression);
            }
        }


        /// <summary>
        /// Fires when documents should be searched.
        /// </summary>
        private void btnSearchDocs_Click(object sender, RoutedEventArgs e)
        {
            // Get search expression
            string expression = txtDocumentName.Text.Trim();

            // Search documents
            SearchDocumentsByASMX(expression);
        }

        #endregion


        #region " Search by ASMX "

        private void SearchByASMX(string expression)
        {
            // Get service full URL within the current web application
            Uri serviceUri = GetServiceUri("CMSWebService.asmx");

            BasicHttpBinding binding = new BasicHttpBinding();
            EndpointAddress endpoint = new EndpointAddress(serviceUri);

            // Create and initialize web service client
            CMSWebService.CMSWebServiceSoapClient client =
                new CMSWebService.CMSWebServiceSoapClient(binding, endpoint);
            client.GetUsersCompleted +=
                new EventHandler<CMSWebService.GetUsersCompletedEventArgs>(client_GetUsersCompleted);

            // Get users
            client.GetUsersAsync(expression);
        }

        void client_GetUsersCompleted(object sender, CMSWebService.GetUsersCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                // Display result after the data are retrieved
                DisplayResult(GetUsers(e.Result));
            }
        }


        #endregion


        #region " Search by WCF "

        private void SearchByWCF(string expression)
        {
            // Get service full URL within the current web application
            Uri serviceUri = GetServiceUri("CMSService.svc");

            BasicHttpBinding binding = new BasicHttpBinding();
            EndpointAddress endpoint = new EndpointAddress(serviceUri);

            // Create and initialize web service client
            CMSService.CMSServiceClient client = new CMSService.CMSServiceClient(binding, endpoint);
            client.GetUsersCompleted +=
                new EventHandler<CMSService.GetUsersCompletedEventArgs>(client_GetUsersCompleted);

            // Get users
            client.GetUsersAsync(expression);
        }

        void client_GetUsersCompleted(object sender, SilverlightDemo.CMSService.GetUsersCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                // Display result after the data are retrieved
                DisplayResult(GetUsers(e.Result));
            }
        }

        #endregion


        #region " Search by ADO.NET Data Services "

        private void SearchByADONet(string expression)
        {
            // Initialize data service
            Uri serviceUri = GetServiceUri("CMSDataService.svc");
            CMSDataService.CMSModelEntities entities = new CMSDataService.CMSModelEntities(serviceUri);

            // Build query to select specified users
            var query = from user in entities.CMS_User select user;

            // Add WHERE condtition
            if (expression != "")
            {
                query = query.Where(user => user.UserName.Contains(expression) ||
                    user.FirstName.Contains(expression) ||
                    user.MiddleName.Contains(expression) ||
                    user.LastName.Contains(expression) ||
                    user.FullName.Contains(expression) ||
                    user.Email.Contains(expression));
            }

            // Get the data
            DataServiceQuery<CMSDataService.CMS_User> userQuery = 
                (DataServiceQuery<CMSDataService.CMS_User>)query;
            userQuery.BeginExecute(new AsyncCallback(OnData), query);
        }

        private void OnData(IAsyncResult e)
        {
            DataServiceQuery<CMSDataService.CMS_User> userQuery =
                (DataServiceQuery<CMSDataService.CMS_User>)e.AsyncState;

            // Display result after the data are retrieved
            DisplayResult(userQuery.EndExecute(e));
        }

        #endregion


        #region " Search by Web client "

        private void SearchByWebClient(string expression)
        {
            // Get service full URL within the current web application
            Uri serviceUri = GetServiceUri("GetUsers.aspx?search=" + expression);

            // Use web client to download users' data asynchronously using the ~/GetUser.aspx service
            WebClient client = new WebClient();
            client.DownloadStringAsync(serviceUri);
            client.DownloadStringCompleted +=
                new DownloadStringCompletedEventHandler(client_DownloadStringCompleted);
        }

        void client_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                // Display result after the data are retrieved
                DisplayResult(GetUsers(e.Result));
            }
        }

        #endregion


        #region " Search by Web request "

        /// <summary>
        /// Delegate which specifies function with one 'IEnumerable' parameter. 
        /// </summary>
        private delegate void ResponseSucceded(IEnumerable users);


        private void SearchByWebRequest(string expression)
        {
            // Get service full URL within the current web application
            Uri serviceUri = GetServiceUri("GetUsers.aspx?search=" + expression);

            // Begin an asynchronous request to the ~/GetUser.aspx service
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(serviceUri);
            request.BeginGetResponse(OnResponseSucceded, request);
        }


        /// <summary>
        /// Fired when data are retrieved successfully.
        /// </summary>
        /// <param name="e">Represents the status of an asynchronous operation.</param>
        private void OnResponseSucceded(IAsyncResult e)
        {
            HttpWebRequest request = (HttpWebRequest)e.AsyncState;
            HttpWebResponse response = (HttpWebResponse)request.EndGetResponse(e);

            // Get the collection of users from response
            StreamReader data = new StreamReader(response.GetResponseStream());
            IEnumerable<UserInfo> users = GetUsers(data.ReadToEnd());

            // Display result - synchronize with UI !!! 
            Dispatcher.BeginInvoke(new ResponseSucceded(DisplayResult), users);
        }

        #endregion


        #region " Helper methods "

        /// <summary>
        /// Displays result of the searching.
        /// </summary>
        /// <param name="items">Collection of the documents which were found.</param>
        private void DisplayResult(IEnumerable users)
        {
            // Set data source 
            list.ItemsSource = users;
            int count = list.Items.Count();

            // Some users were found
            if (count > 0)
            {
                // Display list of users
                list.Visibility = Visibility.Visible;

                // Display number of results
                info.Text = "Found users: " + count;
            }
            // No users were found
            else
            {
                // Hide list of users
                list.Visibility = Visibility.Collapsed;

                // Display info
                info.Text = "No users found.";
            }

            // Hide documents
            listDocs.Visibility = Visibility.Collapsed;
        }


        /// <summary>
        /// Returns collection of users from the specified XML data.
        /// </summary>
        /// <param name="xml">XML containing the users' data.</param>
        private IEnumerable<UserInfo> GetUsers(string xml)
        {
            XDocument users = XDocument.Parse(xml);
            var data = from user in users.Descendants("user")
                       select new UserInfo
                       {
                           FullName = (string)user.Attribute("FullName"),
                           Email = (string)user.Attribute("Email"),
                       };

            return data;
        }


        /// <summary>
        /// Returns URI for the specified relative URL.
        /// </summary>
        /// <param name="relativeUrl">Relative URL.</param>
        private Uri GetServiceUri(string relativeUrl)
        {
            return new Uri(HtmlPage.Document.DocumentUri, relativeUrl);
        }

        #endregion


        #region " Search documents by ASMX "


        private void SearchDocumentsByASMX(string expression)
        {
            // Get service full URL within the current web application
            Uri serviceUri = GetServiceUri("CMSWebService.asmx");

            BasicHttpBinding binding = new BasicHttpBinding();
            EndpointAddress endpoint = new EndpointAddress(serviceUri);

            // Create and initialize web service client
            CMSWebService.CMSWebServiceSoapClient client =
                new CMSWebService.CMSWebServiceSoapClient(binding, endpoint);
            client.GetDocumentsCompleted += 
                new EventHandler<CMSWebService.GetDocumentsCompletedEventArgs>(client_GetDocumentsCompleted);

            // Get documents
            client.GetDocumentsAsync(expression);
        }

        void client_GetDocumentsCompleted(object sender, SilverlightDemo.CMSWebService.GetDocumentsCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                // Display result after the data are retrieved
                DisplayDocuments(GetDocuments(e.Result));
            }
        }


        /// <summary>
        /// Displays result of the searching.
        /// </summary>
        /// <param name="items">Collection of the documents which were found.</param>
        private void DisplayDocuments(IEnumerable documents)
        {
            // Set data source 
            listDocs.ItemsSource = documents;
            int count = listDocs.Items.Count();

            // Some users were found
            if (count > 0)
            {
                // Display list of users
                listDocs.Visibility = Visibility.Visible;

                // Display number of results
                info.Text = "Found documents: " + count;
            }
            // No users were found
            else
            {
                // Hide list of users
                listDocs.Visibility = Visibility.Collapsed;

                // Display info
                info.Text = "No documents found.";
            }

            // Hide users
            list.Visibility = Visibility.Collapsed;
        }


        /// <summary>
        /// Returns collection of documents from the specified XML data.
        /// </summary>
        /// <param name="xml">XML containing the documents' data.</param>
        private IEnumerable<DocumentInfo> GetDocuments(string xml)
        {
            XDocument docs = XDocument.Parse(xml);
            var data = from doc in docs.Descendants("document")
                       select new DocumentInfo
                       {
                           DocumentName = (string)doc.Attribute("DocumentName"),
                       };

            return data;
        }

        #endregion
    }
}

